//
// Copyright (c) 2000-2001 - Andrew Richards <ajr@users.sourceforge.net>
//
// This file is part of the PikView image viewer released under the GNU GPL.
// For license details see the file COPYING included with this distribution.
//


#ifndef KDIRVIEW_H
#define KDIRVIEW_H

#include <qlistview.h>
#include <qstring.h>
#include <qfile.h>
#include <qevent.h>


class KDirViewItem : public QListViewItem
{
	//	Q_OBJECT;

 public:
	KDirViewItem( QListView * parent );
	KDirViewItem( KDirViewItem * parent, const char * filename );
	KDirViewItem( QListView * parent, const char *name );
	
#if QT_VERSION>=200
	QString text(int column) const;
#else
		const char *text(int column) const;
#endif
		
	QString fullName();
	void setOpen( bool );
	void setup();
	void openDir(QString path, int pos=1);
	bool hasSubDirs();

	static void setupIcons();
	static void cleanUpIcons();

 protected:
	void paintCell(QPainter *p, const QColorGroup &cg, int column, int width,
								 int align);
	
private:
	void init();

	QFile f;
	KDirViewItem * p;
	bool readable;

	static QPixmap *closedIcon, *openIcon, *linkIcon, *indexIcon,
		*globalIndexIcon, *bothIndexIcon;
};



class KDirView : public QListView
{
	Q_OBJECT;

	friend KDirViewItem;
 public:
  KDirView(QWidget *parent=0, const char *name=0);
  KDirView(char *dir, QWidget *parent=0, const char *name=0);
  ~KDirView();

	void setRootDir(QString rootDir);
	void setDir(QString startDir, bool emitSignal=true);

 signals:
	void dropAction(QStrList &srcUris, QString destDir, QDropEvent::Action action);
	void dirSelected(QListViewItem*);

 public slots:
	void refresh();
  void slotSelectionChanged(QListViewItem*);

 protected:
	virtual void resizeEvent(QResizeEvent *e);
	virtual void timerEvent(QTimerEvent *e);
 	virtual void dragEnterEvent(QDragEnterEvent *e);
	virtual void dragMoveEvent(QDragMoveEvent *e);
	virtual void dropEvent(QDropEvent *e);
	virtual void dragLeaveEvent(QDragLeaveEvent *e);
	//	virtual void mousePressEvent(QMouseEvent *e);
	void subDirScan();
	KDirViewItem *dropHighlightItem() { return dropHighlight; };
	


 private:
	void init(char *dir=0);
	KDirViewItem *dirScanPos, *dropHighlight;
	int scanTimer;
	bool enableSubDirCheck, haveDropHighlight, emitDirSelected;
};

#endif
