//
// Copyright (c) 2000-2001 - Andrew Richards <ajr@users.sourceforge.net>
//
// This file is part of the PikView image viewer released under the GNU GPL.
// For license details see the file COPYING included with this distribution.
//

#include"config.h"

#define HAVE_MAGICK
#ifdef HAVE_MAGICK

#define _FILE_OFFSET_BITS 64

// #include <Magick++.h>
#include <magick/magick.h>
#include <qimage.h>

// using namespace Magick;

void
kimgio_magick_read(QImageIO *_imageio)
{
	//puts("loading via image magick");
	QImage img;

	// Create an image object and read an image via ImageMagick
	ExceptionInfo exception;
	Image *image;
	ImageInfo *imageInfo;
		
	imageInfo=CloneImageInfo((ImageInfo *) NULL);
	strcpy(imageInfo->filename, _imageio->fileName().ascii());
	image=ReadImage(imageInfo, &exception);
	if (image == (Image *) NULL)  {
#if MagickLibVersion < 0x0527
		fprintf(stderr, "KMagic exception: %s\n",exception.message);
#else
         fprintf(stderr, "KMagic exception: %s: %s\n",
                 exception.reason, exception.description);
#endif
		_imageio->setStatus(1);
		return;
	}
		
	//printf("%d x %d\n",image->columns,image->rows);
	
	//Make the QT image
	img.create(image->columns, image->rows, 32);
		
	if (image->colorspace != RGBColorspace) {
		//puts("Magick transforming colourspace");
		TransformRGBImage(image, RGBColorspace);
	}
	PixelPacket *pixels = GetImagePixels(image, 0,0, image->columns, image->rows);
	
	uchar *imgP=img.bits(), *pixelP;
	pixelP=(uchar*)pixels;
	for(int p=0; p<image->columns*image->rows; p++) {
		*imgP=*(pixelP+3);
		*(imgP+1)=*(pixelP+2);
		*(imgP+2)=*(pixelP+1);
		*(imgP+3)=*(pixelP);
		imgP+=4;
		pixelP+=4;
	}
	//	memcpy(img.bits(), pixels, 4*image->columns*image->rows);

	_imageio->setImage(img);
	_imageio->setStatus(0);
}

// void
// kimgio_magick_rea2(QImageIO *_imageio)
// {
// 	puts("loading via image magick");
// 	QImage img;
	
// 	try {
// 		// Create an image object and read an image

// 		Image image(_imageio->fileName().ascii());
// 		//printf("%d x %d\n",image.columns(),image.rows());
		
// 		img.create(image.columns(), image.rows(), 32);
		
// 		PixelPacket *pixels = image.getPixels(0,0, image.columns(), image.rows());
// 		if (image.colorSpace() != RGBColorspace) {
// 			puts("Magick++ transforming colourspace");
// 			image.colorSpace(RGBColorspace);
// 		}
// 		printf("cs=%d\n",image.colorSpace());
// 		memcpy(img.bits(), pixels, 4*image.columns()*image.rows());
// 	}
// 	catch (Exception err) {
// 		fprintf(stderr, "KMagic exception: %s\n",err.what());
// 		_imageio->setStatus(1);
// 		return;
// 	}
	
// 	_imageio->setImage(img);
// 	_imageio->setStatus(0);
// }

void
kimgio_magick_write(QImageIO *_imageio)
{
}

void
kimgio_magick_register()
{
	// I think only XV puts this kinda header on it
	QImageIO::defineIOHandler("PICT", "^PICSoftware", 0,
														kimgio_magick_read, kimgio_magick_write);

	QImageIO::defineIOHandler("DCX", "\261h\336", 0,
														kimgio_magick_read, kimgio_magick_write);

	QImageIO::defineIOHandler("PCX", "\xa\x5\x1\x8", 0,

														kimgio_magick_read, kimgio_magick_write);

	// ImageMagick Image File Formt
	QImageIO::defineIOHandler("MIF", "^id\\=ImageMagick", 0,
														kimgio_magick_read, kimgio_magick_write);

	//Windows BMP/DIB
	QImageIO::defineIOHandler("BMP", "^BM", 0,
														kimgio_magick_read, kimgio_magick_write);

	// FITS
	QImageIO::defineIOHandler("FITS", "^SIMPLE  =", 0,
														kimgio_magick_read, kimgio_magick_write);

	// Khoros Visualization image file
	QImageIO::defineIOHandler("VIFF", "^\xab", 0,
														kimgio_magick_read, kimgio_magick_write);

	// pgb
	QImageIO::defineIOHandler("PNM", "^P[4-6]", 0,
														kimgio_magick_read, kimgio_magick_write);

	// eps
	QImageIO::defineIOHandler("EPS", "^%!PS-Adobe-", "T",
														kimgio_magick_read, kimgio_magick_write);

	QImageIO::defineIOHandler("PSD", "^8BPS", 0,
														kimgio_magick_read, kimgio_magick_write);

	QImageIO::defineIOHandler("TGA", 0, 0,
														kimgio_magick_read, kimgio_magick_write);

}

#endif
