//
// Copyright (c) 2000-2001 - Andrew Richards <ajr@users.sourceforge.net>
//
// This file is part of the PikView image viewer released under the GNU GPL.
// For license details see the file COPYING included with this distribution.
//


#include <qimage.h>
#include <qregexp.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <dcopclient.h>

#include <stdlib.h>

#include "kapp-safe.h"
#include "kconfig.h"

#include "thumbManager.h"
#include "selector.h"
#include "options.h"

#include <kimageio.h>

#include "misc.h"
#include <config.h>

#include "pv-jpeg.h"

void kimgio_magick_register();

void processCommandLineSpecial();
void processCommandLine();
void rebuildThumbs(QString dir);
void buildThumbs(QString dir);
void packThumbs(QString dir);
void usage();


static KCmdLineOptions options[] = {
	{"build-thumbs <base-dir>",
		 I18N_NOOP("Build thumbnails in <base-dir> XXX"), 0},
	{"rebuild-thumbs <base-dir>",
		 I18N_NOOP("Rebuild thumbnails in <base-dir> XXX"), 0},
	{"pack-thumbs <base-dir>",
	 I18N_NOOP("Pack thumbnail database in <base-dir>"), 0},
	{"w", 0, 0},
	{"viewer", I18N_NOOP("Start in the viewer"), 0},
	{"s", 0, 0},
	{"selector", I18N_NOOP("Start in the selector"), 0},
	{"f", 0, 0},
	{"fullscreen", I18N_NOOP("Start in fullscreen mode"), 0},
	{"l", 0, 0},
	{"filelist", I18N_NOOP("Show the filelist in the file pane"), 0},
	{"t", 0, 0},
	{"thumbnails", I18N_NOOP("Show thumbnails in the file pane"), 0},
	{"i", 0, 0},
	{"image <#>", I18N_NOOP("Start by loading image # on the image list"), 0},
	{"async", I18N_NOOP("Don't use the asyncronous image loader thread"), 0},
	{"+[dir]", I18N_NOOP("Directory to start in"), 0},
	{"+[files]", I18N_NOOP("A list of files to load"), 0},
	{0, 0, 0}
};

void registerImageFormats()
{
	KImageIO::registerFormats();
	kimgio_magick_register();
	QImageIO::defineIOHandler("JPEG","^\377\330", 0,
														PVJPEG::kimgio_jpeg_read, PVJPEG::kimgio_jpeg_write);
}

void
processCommandLineSpecial()
{
	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

	if (args->isSet("rebuild-thumbs"))
		rebuildThumbs(args->getOption("rebuild-thumbs"));
	if (args->isSet("build-thumbs"))
		buildThumbs(args->getOption("build-thumbs"));
	if (args->isSet("pack-thumbs"))
		packThumbs(args->getOption("pack-thumbs"));
}


void
packThumbs(QString dir)
{
	int res=0;
	QString database, dirBase;

	KApplication app(false,false);
	registerImageFormats();
	opt.loadOptions();

	thumbManager *thumbMgr=new thumbManager;
	database=thumbMgr->findThumbNailFile(dir, dirBase);

	if (database.isEmpty()) {
		printf(i18n("PikView: Cannot find thumbnail file for this directory\n"));
		res=1;
	} else {
		printf(i18n("Pack Thumbs:  %s\n"),database.data());
		thumbMgr->packDatabase(database);
	}
	delete thumbMgr;
	exit(res);
}


void
rebuildThumbs(QString dir)
{
	int res=0;
	QString database, dirBase;

	KApplication app(false,false);
	registerImageFormats();
	opt.loadOptions();

	thumbManager *thumbMgr=new thumbManager;
	database=thumbMgr->findThumbNailFile(dir, dirBase);

	if (database.isEmpty()) {
		printf("PikView: Cannot find thumbnail file for this directory\n");
		res=1;
	} else {
		printf("Rebuild Thumbs: %s\n",database.data());
		thumbMgr->rebuildDatabase(database);
	}
	delete thumbMgr;
	exit(res);
}


void
buildThumbs(QString dir)
{
	int res;

	KApplication app(false,false);
	registerImageFormats();
	opt.loadOptions();

	thumbManager *thumbMgr=new thumbManager;
	res=thumbMgr->buildThumbNails(dir);
	delete thumbMgr;
	if (res!=0) {
		printf(i18n("Cannot build thumbnails because the directory specified does not exist or is\nunreadable\n"));
		res=1;
	}
	exit(res);
}


void
processCommandLine()
{
	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

	if (args->isSet("viewer"))
		opt.startInSelector=false;
	if (args->isSet("selector"))
		opt.startInSelector=true;
	if (args->isSet("fullscreen"))
		opt.fullScreen=true;
	if (args->isSet("filelist"))
		opt.fileViewType=0;
	if (args->isSet("thumbnails"))
		opt.fileViewType=1;
	if (args->isSet("image"))
		opt.startImage=args->getOption("image").toInt();
	if (args->isSet("async"))
		opt.useAsyncLoader=false;
}

int main( int argc, char **argv )
{
	KAboutData about("pikview",
									 I18N_NOOP("PikView"),
									 VERSION,
									 I18N_NOOP("A fast feature rich image viewer"),
									 KAboutData::License_GPL_V2,
									 "(c) Andrew Richards 2000-2001.",
									 "pikview.sourceforge.net",
									 I18N_NOOP(""), // more text
									 "ajr@users.sourceforge.net");

	about.addAuthor("Andrew Richards", I18N_NOOP("Creator and Developer"),
									"ajr@users.sourceforge.net",
									"http://pikview.sourceforge.net/");
	
	KCmdLineArgs::init(argc, argv, &about);
	KCmdLineArgs::addCmdLineOptions(options);
	KApplication::addCmdLineOptions();
	processCommandLineSpecial();

	KApplication::setColorSpec( QApplication::ManyColor ); // XXX does what?
	KApplication app;

	registerImageFormats();
	
	// register ourselves as a dcop client
	app.dcopClient()->registerAs(app.name(), false);
	
	opt.loadOptions();
	processCommandLine();

	if (app.isRestored()) {
		fprintf(stderr, i18n("No session management in PikView yet.\n"));
		exit(1);
	}	
	selector *it=new selector();
	app.setMainWidget(it);	
  app.exec();

	return(0);
}
