//
// Copyright (c) 2000-2001 - Andrew Richards <ajr@users.sourceforge.net>
//
// This file is part of the PikView image viewer released under the GNU GPL.
// For license details see the file COPYING included with this distribution.
//

#include <kconfig.h>
#include <qdir.h>
#include "kapp-safe.h"
#include <klocale.h>

#include "options.h"
#include "selector.h"
#include "viewer.h"
#include "types.h"
#include "misc.h"
#include "thumbManager.h"
#include "imageCache.h"

// global options class with a lot of public variables (no acceesors)
options opt;


void
options::loadOptions()
{
	opt.embedded=false;
	loadSelectorOptions();
	loadViewerOptions();
	loadThumbnailOptions();
	loadImageCacheOptions();
}


void
options::loadSelectorOptions()
{
	KConfig *conf=KGlobal::config();
	
  conf->setGroup("Selector Settings");
	opt.selSaveWinPos = conf->readNumEntry("SaveWinPos",true);
	opt.selSaveWinSize = conf->readNumEntry("SaveWinSize",true);
	opt.showStatusBar = conf->readNumEntry("ShowStatusBar",true);
	opt.hideSelector  = conf->readNumEntry("HideSelector",false);
	opt.wrapAround    = conf->readNumEntry("WrapAround",false);
	opt.selWinPosX    = conf->readNumEntry("WinPosX",0);
	opt.selWinPosY    = conf->readNumEntry("WinPosY",0);
	opt.selWinHeight  = conf->readNumEntry("WinHeight",500);
	opt.selWinWidth   = conf->readNumEntry("WinWidth",500);
	opt.hideDirTree   = conf->readNumEntry("HideDirTree",false);
	opt.showHidden    = conf->readNumEntry("ShowHidden",true);
	opt.showNonImageFiles = conf->readNumEntry("ShowNonImageFiles",false);
	opt.startInLastDir= conf->readNumEntry("StartInLastDir",false);
	opt.lastDir       = conf->readEntry("LastDir",".");
	opt.fileViewType  = conf->readNumEntry("FileViewType",0);
	opt.treeDirWidth  = conf->readNumEntry("TreeDirWidth",200);
	opt.confirmDelete = conf->readNumEntry("ConfirmDelete",true);

	if (!QDir(opt.lastDir).exists())
		opt.lastDir=".";
	opt.startImage=0;
}


void
options::saveSelectorOptions(selector *sel, bool exiting=false)
{
	//puts("options::saveSelectorOptions");

	KConfig *conf=KGlobal::config();

  conf->setGroup("Selector Settings");
	conf->writeEntry("StartInLastDir",opt.startInLastDir);
	conf->writeEntry("LastDir",sel->currentDir);
	conf->writeEntry("HideDirTree",opt.hideDirTree);
	conf->writeEntry("ShowHidden",opt.showHidden);
	conf->writeEntry("ShowNonImageFiles",opt.showNonImageFiles);
	conf->writeEntry("WrapAround",opt.wrapAround);
	conf->writeEntry("FileViewType",opt.fileViewType);
	opt.treeDirWidth=sel->treeDirWidth();
	conf->writeEntry("TreeDirWidth",opt.treeDirWidth);
	conf->writeEntry("ShowStatusBar",opt.showStatusBar);
	conf->writeEntry("ConfirmDelete",opt.confirmDelete);

	conf->writeEntry("SaveWinPos",opt.selSaveWinPos);
	conf->writeEntry("SaveWinSize",opt.selSaveWinSize);

	if (opt.selSaveWinPos) {
		if (sel->isHidden()) {
			conf->writeEntry("WinPosX",opt.selWinPosX);
			conf->writeEntry("WinPosY",opt.selWinPosY);
		} else {
			conf->writeEntry("WinPosX",sel->pos().x());
			conf->writeEntry("WinPosY",sel->pos().y());
		}
	}
	if (opt.selSaveWinSize) {
		conf->writeEntry("WinHeight",sel->height());
		conf->writeEntry("WinWidth",sel->width());
	}
	conf->sync();
	//puts("finished: saveSelectorOptions");
}


void
options::loadViewerOptions()
{
	KConfig *conf=KGlobal::config();

	opt.fullScreen    = false;

  conf->setGroup("Viewer Settings");
	opt.centreWindow  = conf->readNumEntry("CentreWindow",true);
	opt.hideMouse     = conf->readNumEntry("HideMouse",false);
	opt.imageInfoInFS = conf->readNumEntry("ImageInfoInFS",true);
	opt.resizeWindow  = conf->readNumEntry("ResizeWindow",1);
	opt.zoomToFit     = conf->readNumEntry("ZoomToFit",false);
	opt.shrinkToFit   = conf->readNumEntry("ShrinkToFit",false);
	opt.viewerSaveWinSize= conf->readNumEntry("SaveWinSize",true);
	opt.viewerSaveWinPos = conf->readNumEntry("SaveWinPos",true);
	opt.viewerWinPosX = conf->readNumEntry("WinPosX",0);
	opt.viewerWinPosY = conf->readNumEntry("WinPosY",0);
	opt.viewerWinHeight= conf->readNumEntry("WinHeight",500);
	opt.viewerWinWidth= conf->readNumEntry("WinWidth",500);
	opt.slideShowDelay= conf->readNumEntry("SlideShowDelay",1000);	
	opt.slideShowDir  = conf->readNumEntry("SlideShowDir",1);
	opt.showSelectorAndViewer=conf->readNumEntry("ShowSelectorAndViewer",false);
	opt.saveViewerSettingsOnExit=conf->readNumEntry("SaveSettingsOnExit",true);
	opt.startInSelector=conf->readNumEntry("StartInSelector",false);
	
	int i1=conf->readListEntry("ShortCuts", opt.shortCuts);
	int i2=conf->readListEntry("ShortCutDirs", opt.shortCutDirs);
	if (i1!=i2) {
		puts("Invalid shortcut list - clearing");
		opt.shortCuts.clear();
		opt.shortCutDirs.clear();
	}
	opt.enableFiling=conf->readNumEntry("EnableFiling", true);
	opt.useAsyncLoader=conf->readNumEntry("UseAsyncLoader", true);
}


void
options::saveShortCuts()
{
	KConfig *conf=KGlobal::config();

  conf->setGroup("Viewer Settings");
	conf->writeEntry("ShortCuts", opt.shortCuts);
	conf->writeEntry("ShortCutDirs", opt.shortCutDirs);
	conf->writeEntry("EnableFiling", opt.enableFiling);
	conf->sync();
}


void
options::saveViewerOptions(viewer *view, bool exiting=false)
{
	KConfig *conf=KGlobal::config();

  conf->setGroup("Viewer Settings");
	conf->writeEntry("SaveWinSize",opt.viewerSaveWinSize);
	if (opt.viewerSaveWinSize) {
		if (opt.fullScreen) {
			conf->writeEntry("WinHeight",opt.viewerWinHeight);
			conf->writeEntry("WinWidth",opt.viewerWinWidth);			
		} else {
			conf->writeEntry("WinHeight",view->height());
			conf->writeEntry("WinWidth",view->width());
		}
	}
	conf->writeEntry("SaveWinPos",opt.viewerSaveWinPos);
	if (opt.viewerSaveWinPos) {
		if (view->isHidden() || opt.fullScreen) {
			conf->writeEntry("WinPosX",opt.viewerWinPosX);
  		conf->writeEntry("WinPosY",opt.viewerWinPosY);
		} else {
			conf->writeEntry("WinPosX", view->pos().x());
			conf->writeEntry("WinPosY", view->pos().y());
		}
	}
	conf->writeEntry("SaveSettingsOnExit",opt.saveViewerSettingsOnExit);
	if (opt.saveViewerSettingsOnExit) {
	  conf->writeEntry("CentreWindow",opt.centreWindow);
		conf->writeEntry("ImageInfoInFS",opt.imageInfoInFS);
		conf->writeEntry("HideMouse",opt.hideMouse);
	  conf->writeEntry("ResizeWindow",opt.resizeWindow);
		conf->writeEntry("ZoomToFit",opt.zoomToFit);
		conf->writeEntry("ShrinkToFit",opt.shrinkToFit);
	}
	conf->writeEntry("SlideShowDelay",opt.slideShowDelay);	
	conf->writeEntry("SlideShowDir",opt.slideShowDir);
	conf->writeEntry("ShowSelectorAndViewer",opt.showSelectorAndViewer);
	conf->writeEntry("StartInSelector",opt.startInSelector);

	conf->sync();
	//puts("viewer::saveSettings - saved stuff");
}

void
options::loadThumbnailOptions()
{	
	KConfig *conf=KGlobal::config();

  conf->setGroup("Thumbnail Settings");
	opt.thumbSizeX = conf->readNumEntry("thumbSizeX", 75);
	opt.thumbSizeY = conf->readNumEntry("thumbSizeY", 75);
	opt.useCache   = conf->readNumEntry("useCache", true);
	opt.textSpaceY = conf->readNumEntry("thumbSpaceY", 20);
	opt.thumbPadX  = conf->readNumEntry("thumbPadX", 10);
	opt.thumbPadY  = conf->readNumEntry("thumbPadY", 10);
}


void
options::saveThumbnailOptions(thumbWidget *sel, bool exiting=false)
{
	KConfig *conf=KGlobal::config();

  conf->setGroup("Thumbnail Settings");
	conf->writeEntry("thumbSizeX", opt.thumbSizeX);
	conf->writeEntry("thumbSizeY", opt.thumbSizeY);
	conf->writeEntry("useCache", opt.useCache);
	conf->writeEntry("thumbSpaceY", opt.textSpaceY);
	conf->writeEntry("thumbPadX", opt.thumbPadX);
	conf->writeEntry("thumbPadY", opt.thumbPadY);
}

void
options::loadImageCacheOptions()
{	
	KConfig *conf=KGlobal::config();

  conf->setGroup("Image Cache Settings");
	opt.imageCacheSize = conf->readNumEntry("imageCacheSize", 4000);
	opt.maxLookAhead   = conf->readNumEntry("maxLookAhead", 4);
}


void
options::saveImageCacheOptions(imageCache *cache, bool exiting=false)
{
	KConfig *conf=KGlobal::config();

  conf->setGroup("Image Cache Settings");
	conf->writeEntry("imageCacheSize", opt.imageCacheSize);
	conf->writeEntry("maxLookAhead", opt.maxLookAhead);
}


void
options::doDialog(viewer *view, selector *sel, thumbWidget *thumb,
									imageCache *cache)
{
	optionsDialog optsDlg(view, sel, thumb, cache);

	if (optsDlg.exec())
		optsDlg.applyOptions();
}


// Options dialog box

optionsDialog::optionsDialog(viewer *view, selector *sel, thumbWidget *thumb,
														 imageCache *cache, QWidget *parent=0,
														 const char *name=0, bool modal=true, int wFlags=0)
	: QTabDialog(parent, name, modal, wFlags)
{
	theViewer=view;
	theSelector=sel;
	theThumb=thumb;
	theCache=cache;

	setCancelButton();

	QWidget *viewTab = new QWidget(this);
	setupViewerTab(viewTab);
	addTab(viewTab, i18n("Viewer"));
// 	connect(btViewerSaveNow, SIGNAL(clicked()), this, SLOT(viewerSaveNow()));

	QWidget *selectTab = new QWidget(this);
	setupSelectorTab(selectTab);
	addTab(selectTab, i18n("Selector"));
// 	connect(btSelSaveNow, SIGNAL(clicked()), this, SLOT(selectorSaveNow()));

	QWidget *slideTab = new QWidget(this);
	setupSlideshowTab(slideTab);
	addTab(slideTab, i18n("Slideshow"));

	QWidget *thumbTab = new QWidget(this);
	setupThumbnailTab(thumbTab);
	addTab(thumbTab, i18n("Thumbnails"));

	QWidget *cacheTab = new QWidget(this);
	setupImageCacheTab(cacheTab);
	addTab(cacheTab, i18n("Image cache"));

	resize(500,290);
	setCaption(i18n("Option settings..."));
}


void
optionsDialog::setupViewerTab(QWidget *w)
{
	QButtonGroup* bg1;
	bg1 = new QButtonGroup(i18n("Viewer settings"), w );
	bg1->setGeometry( 10, 20, 220, 180 );
	bg1->setFrameStyle( 49 );

	cbCentre = new QCheckBox(i18n("Centre window"), w);
	cbCentre->setGeometry( 30, 40, 107, 19 );
	cbCentre->setChecked(opt.centreWindow);
	cbCentre->setAutoResize( TRUE );

	cbImageInfoInFS = new QCheckBox(i18n("Image info in fullscreen"), w );
	cbImageInfoInFS->setGeometry( 30, 60, 83, 19 );
	cbImageInfoInFS->setChecked(opt.imageInfoInFS);
	cbImageInfoInFS->setAutoResize( TRUE );

	cbHideMouse = new QCheckBox(i18n("Hide mouse in fullscreen"), w);
	cbHideMouse->setGeometry( 30, 80, 162, 19 );
	cbHideMouse->setChecked(opt.hideMouse);
	cbHideMouse->setAutoResize( TRUE );

	cbResizeWindow = new QCheckBox(i18n("Resize window"), w);
	cbResizeWindow->setGeometry( 30, 100, 107, 19 );
	cbResizeWindow->setChecked(opt.resizeWindow);
	cbResizeWindow->setAutoResize( TRUE );

	cbZoom = new QCheckBox(i18n("Zoom image to fit"), w);
	cbZoom->setGeometry( 30, 120, 120, 19 );
	cbZoom->setChecked(opt.zoomToFit);
	cbZoom->setAutoResize( TRUE );

	cbShrink = new QCheckBox(i18n("Shrink image to fit"), w);
	cbShrink->setGeometry( 30, 140, 123, 19 );
	cbShrink->setChecked(opt.shrinkToFit);
	cbShrink->setAutoResize( TRUE );

	cbShowSV = new QCheckBox(i18n("Show viewer and selector\n  in windowed mode"), w);
	cbShowSV->setGeometry( 30, 160, 123, 19 );
	cbShowSV->setChecked(opt.showSelectorAndViewer);
	cbShowSV->setAutoResize( TRUE );

	QButtonGroup* bg2;
	bg2 = new QButtonGroup( w );
	bg2->setGeometry( 250, 20, 170, 90 );
	bg2->setFrameStyle( 49 );
	bg2->setTitle(i18n("Save on exit"));
	bg2->setAlignment( 1 );

	cbViewWindowPos = new QCheckBox(i18n("Window position"), w );
	cbViewWindowPos->setGeometry( 270, 40, 115, 20 );
	cbViewWindowPos->setChecked(opt.viewerSaveWinPos);
	cbViewWindowPos->setAutoResize( TRUE );

	cbViewWindowSize = new QCheckBox(i18n("Window size"), w);
	cbViewWindowSize->setGeometry( 270, 60, 94, 19 );
	cbViewWindowSize->setChecked(opt.viewerSaveWinSize);
	cbViewWindowSize->setAutoResize( TRUE );

	cbViewerSettings = new QCheckBox(i18n("Viewer settings"), w);
	cbViewerSettings->setGeometry( 270, 80, 109, 19 );
	cbViewerSettings->setChecked(opt.saveViewerSettingsOnExit);
	cbViewerSettings->setAutoResize( TRUE );

	QButtonGroup* bgStartIn;
	bgStartIn = new QButtonGroup(i18n("Start in"), w);
	bgStartIn->setGeometry(250,130, 100, 70);

	rbStartInViewer = new QRadioButton(i18n("Viewer"), w);
	rbStartInViewer->setGeometry(270, 150, 70,19);
	rbStartInViewer->setChecked(!opt.startInSelector);

	rbStartInSelector = new QRadioButton(i18n("Selector"), w);
	rbStartInSelector->setGeometry(270, 170, 70,19);
	rbStartInSelector->setChecked(opt.startInSelector);

	bgStartIn->insert(rbStartInViewer);
	bgStartIn->insert(rbStartInSelector);

	bg1->insert( cbZoom );
	bg1->insert( cbResizeWindow );
	bg1->insert( cbImageInfoInFS );
	bg1->insert( cbHideMouse );
	bg1->insert( cbCentre );
	bg1->insert( cbShrink );
	bg1->insert( cbShowSV );

	bg2->insert( cbViewWindowSize );
	bg2->insert( cbViewWindowPos );
	bg2->insert( cbViewerSettings );

	resize( 470,220 );
}


void
optionsDialog::setupSelectorTab(QWidget *w)
{
	QButtonGroup* bg2;
	bg2 = new QButtonGroup( w );
	bg2->setGeometry( 10, 20, 220, 170 );
	bg2->setFrameStyle( 49 );
	bg2->setTitle( i18n("File settings") );
	bg2->setAlignment( 1 );

	cbStartInLastDir = new QCheckBox(i18n("Startup in previous directory"), w);
	cbStartInLastDir->setGeometry( 30, 40, 182, 19 );
	cbStartInLastDir->setChecked(opt.startInLastDir);
	cbStartInLastDir->setAutoResize( TRUE );

	cbHideDirTree = new QCheckBox(i18n("Hide directory tree"), w);
	cbHideDirTree->setGeometry( 30, 60, 128, 19 );
	cbHideDirTree->setChecked(opt.hideDirTree);
	cbHideDirTree->setAutoResize( TRUE );

	cbShowHidden = new QCheckBox(i18n("Show hidden files"), w);
	cbShowHidden->setGeometry( 30, 80, 123, 19 );
	cbShowHidden->setChecked(opt.showHidden);
	cbShowHidden->setAutoResize( TRUE );

	cbShowNonImageFiles = new QCheckBox(i18n("Show non-image files"), w);
	cbShowNonImageFiles->setGeometry( 30, 100, 147, 19 );
	cbShowNonImageFiles->setChecked(opt.showNonImageFiles);
	cbShowNonImageFiles->setAutoResize( TRUE );

	cbWrapFiles = new QCheckBox(i18n("Wrap around filelist"), w);
	cbWrapFiles->setGeometry( 30, 120, 131, 19 );
	cbWrapFiles->setChecked(opt.wrapAround);
	cbWrapFiles->setAutoResize( TRUE );

	cbShowStatusBar = new QCheckBox(i18n("Show status bar"), w);
	cbShowStatusBar->setGeometry( 30, 140, 131, 19 );
	cbShowStatusBar->setChecked(opt.showStatusBar);
	cbShowStatusBar->setAutoResize( TRUE );

	QLabel *lbl;
	lbl=new QLabel(i18n("File view:"),w);
	lbl->move(30,165);
	lbl->setAutoResize( true );
	cbbViewType=new QComboBox(w);
	cbbViewType->setAutoResize( true );
	cbbViewType->move(100,160);
	cbbViewType->insertItem(i18n("File list"));
	cbbViewType->insertItem(i18n("Thumbnails"));
	cbbViewType->setCurrentItem(opt.fileViewType);

	QButtonGroup* bg1;
	bg1 = new QButtonGroup(i18n("Save on exit"), w);
	bg1->setGeometry( 250, 20, 170, 130 );
	bg1->setFrameStyle( 49 );
	bg1->setAlignment( 1 );

	cbSelWindowPos = new QCheckBox(i18n("Window position"), w);
	cbSelWindowPos->setGeometry( 270, 40, 115, 20 );
	cbSelWindowPos->setChecked(opt.selSaveWinPos);
	cbSelWindowPos->setAutoResize( TRUE );

	cbSelWindowSize = new QCheckBox(i18n("Window size"), w);
	cbSelWindowSize->setGeometry( 270, 60, 94, 19 );
	cbSelWindowSize->setChecked(opt.selSaveWinSize);
	cbSelWindowSize->setAutoRepeat( FALSE );

	bg1->insert( cbSelWindowSize );
	bg1->insert( cbSelWindowPos );

	bg2->insert( cbStartInLastDir );
	bg2->insert( cbHideDirTree );
	bg2->insert( cbShowNonImageFiles );
	bg2->insert( cbShowHidden );
	bg2->insert( cbWrapFiles );
	bg2->insert( cbShowStatusBar );

	resize( 470,220 );
}


void
optionsDialog::setupSlideshowTab(QWidget *w)
{
	QLabel* lbl;
	lbl = new QLabel(i18n("Flip delay (ms)"), w);
	lbl->setGeometry( 20, 30, 84, 15 );
	lbl->setAlignment( 289 );
	lbl->setMargin( -1 );

	sbDelay = new QSpinBox( 0,10000, 500, w );
	sbDelay->setGeometry( 120, 30, 60, 20 );
	sbDelay->setValue(opt.slideShowDelay);

	slDelay = new QSlider(0,10000, 10, opt.slideShowDelay,
												QSlider::Horizontal, w);
	slDelay->setGeometry( 190, 30, 270, 20 );
	//	slDelay->setSteps( 100, 1000 );
	slDelay->setTracking( TRUE );
	slDelay->setTickmarks( QSlider::Both );
	slDelay->setTickInterval( 1000 );

	connect(slDelay, SIGNAL(sliderMoved(int)), sbDelay, SLOT(setValue(int)));
	connect(sbDelay, SIGNAL(valueChanged(int)), slDelay, SLOT(setValue(int)));

	QButtonGroup* bg7;
	bg7 = new QButtonGroup(i18n("Direction"), w);
	bg7->setGeometry( 20, 70, 110, 90 );
	bg7->setFrameStyle( 49 );
	bg7->setAlignment( 1 );

	rbDirForward = new QRadioButton(i18n("Forward"), w);
	rbDirForward->setGeometry( 40, 90, 68, 19 );
	rbDirForward->setChecked(opt.slideShowDir==1);
	rbDirForward->setAutoResize( TRUE );

	rbDirReverse = new QRadioButton(i18n("Reverse"), w);
	rbDirReverse->setGeometry( 40, 110, 68, 19 );
	rbDirReverse->setChecked(opt.slideShowDir==-1);
	rbDirReverse->setAutoResize( TRUE );

	rbDirRandom = new QRadioButton(i18n("Random"), w);
	rbDirRandom->setGeometry( 40, 130, 67, 19 );
	rbDirRandom->setChecked(opt.slideShowDir==0);
	rbDirRandom->setAutoResize( TRUE );

	bg7->insert( rbDirForward );
	bg7->insert( rbDirRandom );
	bg7->insert( rbDirReverse );

	resize( 470,220 );
}


void
optionsDialog::setupThumbnailTab(QWidget *w)
{
	cgUseCache=new QCheckBox(i18n("Cache thumbnails"),w);
	cgUseCache->setChecked(opt.useCache);
	cgUseCache->move(20, 20);
	cgUseCache->setAutoResize(true);

	const int minX=40, minY=40, maxX=100, maxY=100;
	const int thOffX=20, thOffY=60;

	slThumbSizeY=new QSlider(-maxY, -minY, 10, -opt.thumbSizeY,
													 QSlider::Vertical, w);
	slThumbSizeY->setGeometry(thOffX, thOffY+5, 20, maxY);

	slThumbSizeX=new QSlider(minX,maxX, 10, opt.thumbSizeX,
													 QSlider::Horizontal, w);
	slThumbSizeX->setGeometry(thOffX+25, 5+thOffY+maxY, maxX, 20);

	frThumb=new QFrame(w);
	frThumb->setLineWidth(3);
	frThumb->setFrameStyle(QFrame::Panel | QFrame::Raised);
	frThumb->move(40, 60);

	updateThumbFrame();

	connect(slThumbSizeX, SIGNAL(valueChanged(int)),
					this, SLOT(updateThumbFrame()));
	connect(slThumbSizeY, SIGNAL(valueChanged(int)),
					this, SLOT(updateThumbFrame()));
	

	resize( 470,220 );
}


void
optionsDialog::updateThumbFrame()
{
	const int maxX=100, maxY=100;
	const int thOffX=20, thOffY=60;

	frThumb->move(25+thOffX+(maxX-slThumbSizeX->value())/2,
								thOffY+(maxY+slThumbSizeY->value())/2);
	frThumb->resize(slThumbSizeX->value(), -slThumbSizeY->value());	
}

void
optionsDialog::setupImageCacheTab(QWidget *w)
{
	QLabel *lbl;

	lbl=new QLabel(i18n("Max look ahead depth"), w);
	lbl->setAutoResize(true);
	lbl->move(20,30);

	sbMaxLookAhead=new QSpinBox(0,99, 1, w);
	sbMaxLookAhead->setValue(opt.maxLookAhead);
	sbMaxLookAhead->setGeometry(170, 30, 50,20);

	lbl=new QLabel(i18n("Image cache size (in k)"), w);
	lbl->setAutoResize(true);
	lbl->move(20,60);

	sbImageCacheSize=new QSpinBox(0,99990, 100, w);
	sbImageCacheSize->setValue(opt.imageCacheSize);
	sbImageCacheSize->setGeometry(170, 60, 50,20);

	resize( 470,220 );
}


void
optionsDialog::applyOptions()
{
	applyViewerOptions();
	applySelectorOptions();
	applySlideshowOptions();
	applyThumbnailOptions();
	applyImageCacheOptions();
}


void
optionsDialog::applyViewerOptions()
{
	bool redrawImage=false;

	opt.startInSelector=rbStartInSelector->isChecked();
	opt.viewerSaveWinSize=cbViewWindowSize->isChecked();
	opt.viewerSaveWinPos=cbViewWindowPos->isChecked();
	opt.imageInfoInFS=cbImageInfoInFS->isChecked();

	if (opt.zoomToFit!=cbZoom->isChecked()) {
		opt.zoomToFit=cbZoom->isChecked();
		redrawImage=true;
	}
	if (opt.resizeWindow!=cbResizeWindow->isChecked()) {
		opt.resizeWindow=cbResizeWindow->isChecked();
		redrawImage=true;
	}

	if (opt.centreWindow!=cbCentre->isChecked())
		theViewer->setCentering(toggle);

	if (opt.shrinkToFit!=cbShrink->isChecked()) {
		opt.shrinkToFit=cbShrink->isChecked();
		redrawImage=true;
	}
	opt.saveViewerSettingsOnExit=cbViewerSettings->isChecked();

	if (opt.showSelectorAndViewer!=cbShowSV->isChecked()) {
		opt.showSelectorAndViewer=cbShowSV->isChecked();
		if (opt.showSelectorAndViewer) {
			theSelector->setVisible(on);
			theViewer->show();
		} else {
			theSelector->setVisible(on);
		}
	}

	opt.hideMouse=cbHideMouse->isChecked();

	if (redrawImage)
		theViewer->newImageSetup();
}


void
optionsDialog::applySelectorOptions()
{
	bool reload=false;

	opt.startInLastDir=cbStartInLastDir->isChecked();
	opt.selSaveWinSize=cbSelWindowSize->isChecked();
	opt.selSaveWinPos=cbSelWindowPos->isChecked();
	opt.wrapAround=cbWrapFiles->isChecked();

	if (opt.showNonImageFiles!=cbShowNonImageFiles->isChecked())
		reload=true;
	opt.showNonImageFiles=cbShowNonImageFiles->isChecked();

	if(opt.showHidden!=cbShowHidden->isChecked())
		reload=true;
	opt.showHidden=cbShowHidden->isChecked();

	if (opt.hideDirTree!=cbHideDirTree->isChecked())
		if (cbHideDirTree->isChecked())
			theSelector->hideDirTree();
		else
			theSelector->showDirTree();

	if (opt.showStatusBar!=cbShowStatusBar->isChecked())
		theSelector->setStatusBarVisible(toggle);

	if (opt.fileViewType!=cbbViewType->currentItem())
		if (cbbViewType->currentItem()==0)
			theSelector->setFileListView();
		else
			theSelector->setThumbNailView();

	if (reload && (theSelector->imagesLoadedFrom==selector::directory))
		theSelector->loadAllImages(theSelector->currentDir);	
}


void
optionsDialog::applySlideshowOptions()
{
	opt.slideShowDelay=sbDelay->value();
	if (rbDirForward->isChecked())
		opt.slideShowDir=1;
	if (rbDirReverse->isChecked())
		opt.slideShowDir=-1;
	if (rbDirRandom->isChecked())
		opt.slideShowDir=0;
}


void
optionsDialog::applyThumbnailOptions()
{
	opt.thumbSizeX=slThumbSizeX->value();
	opt.thumbSizeY=-slThumbSizeY->value();
	opt.useCache=cgUseCache->isChecked();
	thumbMgr->setUseDatabase(opt.useCache);
}


void
optionsDialog::applyImageCacheOptions()
{
	opt.maxLookAhead=sbMaxLookAhead->value();
	opt.imageCacheSize=sbImageCacheSize->value();
	theCache->setMaxCost(1024*opt.imageCacheSize);
}
