//
// Copyright (c) 2000 - Andrew Richards <ajr@users.sourceforge.net>
//
// This file is part of the PikView image viewer released under the GNU LPL.
// For license details see the file COPYING included with this distribution.
//


#ifndef OPTIONS_H
#define OPTIONS_H

#include <qtabdialog.h>
#include <qcheckbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qslider.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qstrlist.h>


class viewer;
class selector;
class options;
class thumbWidget;
class imageCache;
extern options opt;

class options : public QObject
{
	Q_OBJECT;

 public:
	void loadOptions();
	void doDialog(viewer *view, selector* sel, thumbWidget *thumb,
								imageCache *cache);

 public slots:
	void loadViewerOptions();
	void saveViewerOptions(viewer *view, bool exiting=false);
	void loadSelectorOptions();
	void saveSelectorOptions(selector *sel, bool exiting=false);
	void loadThumbnailOptions();
	void saveThumbnailOptions(thumbWidget *thumb, bool exiting=false);
	void loadImageCacheOptions();
	void saveImageCacheOptions(imageCache *cache, bool exiting=false);
	void saveShortCuts();
	
 public:

	// Selector Configuration options
	bool wrapAround, showHidden, showNonImageFiles, startInLastDir,
		selSaveWinPos, selSaveWinSize, hideDirTree, hideSelector, showStatusBar,
		startInSelector, confirmDelete;
	int selWinPosX, selWinPosY, selWinWidth, selWinHeight, fileViewType,
		treeDirWidth, startImage;
	QString lastDir;

	// Viewer Configuration options
	bool imageInfoInFS, fullScreen, resizeWindow, zoomToFit, shrinkToFit,
		centreWindow, hideMouse,	viewerSaveWinSize, viewerSaveWinPos,
		saveViewerSettingsOnExit, showSelectorAndViewer;
	int viewerWinPosX, viewerWinPosY,	viewerWinWidth, viewerWinHeight,
		slideShowDelay,	slideShowDir;
	QStrList shortCuts, shortCutDirs;

	// Thumbnail
	bool useCache;
	int thumbSizeX, thumbSizeY, thumbPadX, thumbPadY, textSpaceY;

  // Image cache
	int maxLookAhead, imageCacheSize;

	// rapid filing
	bool enableFiling;

	// Async loader (using threads)
	bool useAsyncLoader;

	// Misc
	bool embedded;
};


class optionsDialog : public QTabDialog
{
	Q_OBJECT;

 public:
	optionsDialog(viewer *view, selector *sel, thumbWidget *thumb,
								imageCache *cache, QWidget *parent=0,
								const char *name=0, bool modal=true, int wFlags=0 );	
	void applyOptions();

 signals:

 private slots:
	void updateThumbFrame();

 private:
	void setupViewerTab(QWidget *w);
	void setupSelectorTab(QWidget *w);
	void setupSlideshowTab(QWidget *w);
	void setupThumbnailTab(QWidget *w);
	void setupImageCacheTab(QWidget *w);
	void applyViewerOptions();
	void applySelectorOptions();
	void applySlideshowOptions();
	void applyThumbnailOptions();
	void applyImageCacheOptions();

	viewer      *theViewer;
	selector    *theSelector;
	thumbWidget *theThumb;
	imageCache  *theCache;

	// Thumbnails
	QSlider   *slThumbSizeX;
	QSlider   *slThumbSizeY;
	QFrame    *frThumb;
	QCheckBox *cgUseCache;

	//Viewer
	QCheckBox* cbViewWindowSize;
	QCheckBox* cbViewWindowPos;
	QCheckBox* cbZoom;
	QCheckBox* cbResizeWindow;
	QCheckBox* cbImageInfoInFS;
	QCheckBox* cbHideMouse;
	QCheckBox* cbCentre;
	QCheckBox* cbShrink;
	QCheckBox* cbShowSV;
	QCheckBox* cbViewerSettings;
	QPushButton* btViewerSaveNow;
	QRadioButton* rbStartInViewer;
	QRadioButton* rbStartInSelector;

	// Selector
	QCheckBox* cbStartInLastDir;
	QCheckBox* cbHideDirTree;
	QCheckBox* cbSelWindowSize;
	QCheckBox* cbSelWindowPos;
	QCheckBox* cbShowNonImageFiles;
	QCheckBox* cbShowHidden;
	QCheckBox* cbWrapFiles;
	QCheckBox *cbShowStatusBar;
	QPushButton* btSelSaveNow;
	QComboBox *cbbViewType;

	// Slideshow
	QSlider* slDelay;
	QSpinBox* sbDelay;
	QRadioButton* rbDirForward;
	QRadioButton* rbDirRandom;
	QRadioButton* rbDirReverse;

	// Image cache
	QSpinBox *sbImageCacheSize;
	QSpinBox *sbMaxLookAhead;
};

#endif
