//
// Copyright (c) 2000-2001 - Andrew Richards <ajr@users.sourceforge.net>
//
// This file is part of the PikView image viewer released under the GNU GPL.
// For license details see the file COPYING included with this distribution.
//

#include <config.h>
#include "pikview_part.h"
#include "viewer.h"

#include <kinstance.h>
#include <klocale.h>
#include <kaboutdata.h>
#include <qlabel.h>

#include "options.h"

extern "C"
{
	/**
	 * This function is the 'main' function of this part.  It takes
	 * the form 'void *init_lib<library name>()  It always returns a
	 * new factory object
	 */
	void *init_libpikview()
	{
		return new PikViewFactory;
	}
};

/**
 * We need one static instance of the factory for our C 'main'
 * function
 */
KInstance *PikViewFactory::s_instance = 0L;

PikViewFactory::PikViewFactory()
{
}

PikViewFactory::~PikViewFactory()
{
	if (s_instance)
		delete s_instance;

	s_instance = 0;
}

QObject *PikViewFactory::create(QObject *parent, const char *name, const char*,
																const QStringList& )
{
	QObject *obj = new PikViewPart((QWidget*)parent, name);
	emit objectCreated(obj);
	return obj;
}

KInstance *PikViewFactory::instance()
{
	if ( !s_instance )
    {
			KAboutData about("pikview", I18N_NOOP("pikview"), VERSION);
			s_instance = new KInstance(&about);
    }
	return s_instance;
}

PikViewPart::PikViewPart(QWidget *parent, const char *name)
	: KParts::ReadOnlyPart(parent, name)
{
	setInstance(PikViewFactory::instance());


	m_extension = new PikViewBrowserExtension(this);

	opt.viewerSaveWinPos=false;
	opt.viewerWinWidth=0;
	opt.viewerWinHeight=0;
	opt.fullScreen=false;
	opt.zoomToFit=false;
	opt.shrinkToFit=false;
	opt.resizeWindow=false;
	opt.centreWindow=true;
	opt.slideShowDelay=1000;
	opt.slideShowDir=999;
	
	widget = new viewer(0, parent);
	widget->setFocusPolicy(QWidget::StrongFocus);
	setWidget(widget);
}

PikViewPart::~PikViewPart()
{
	//closeURL();
}

bool PikViewPart::openFile()
{
	pix.load(m_file);
	widget->newImage(pix, m_file);

	return true;
}

// bool PikViewPart::closeURL()
// {
// 	return true;
// }

PikViewBrowserExtension::PikViewBrowserExtension(PikViewPart *parent)
	: KParts::BrowserExtension(parent, "PikViewBrowserExtension")
{
}

PikViewBrowserExtension::~PikViewBrowserExtension()
{
}

