////////////////////////////////////////////////////
//
// Transparent support for JPEG files in Qt Pixmaps,
// using IJG JPEG library.
//
// Sirtaj Kang, Oct 1996.
//
// Andrew Richards Oct 1998 - bugfix, error handler, write support
//
// $Id: jpeg.cpp,v 1.7 2000/03/20 05:26:56 cvs Exp $

#ifdef HAVE_CONFIG_H
#include"config.h"
#endif

#ifdef HAVE_LIBJPEG

#include<stdio.h>
#include<assert.h>
#include<setjmp.h>

#include"kimageio.h"
#include"qimage.h"
#include"qdstream.h"
#include"qcolor.h"
#include"qpixmap.h"
#include"pv-jpeg.h"


//
// Hack to fix INT16 redefinition problem
//

#define XMD_H
typedef short INT16;
extern "C" {
#include<jpeglib.h>
//XXX
}
#undef XMD_H

namespace PVJPEG {

// allocate space for static variables
int jpegQuality;
int jpegSmoothing;
int jpegOptimizeCoding;

// Buffer for file reads (max bytes)
// ?? Is this good enough? Bigger, perhaps?

#define INPUT_BUFFER_SIZE  4096
#define OUTPUT_BUFFER_SIZE 4096
#define JPEG_GRAY 64

//
// Plug-in methods for JPEG library
//

void kimgio_jpeg_write(QImageIO *iio);
void kimgio_jpeg_init_destination(j_compress_ptr cinfo);
int  kimgio_jpeg_empty_output_buffer(j_compress_ptr cinfo);
void kimgio_jpeg_term_destination(j_compress_ptr cinfo);

void qimageio_init_source(j_decompress_ptr cinfo);
int  qimageio_fill_input_buffer(j_decompress_ptr cinfo);
void qimageio_skip_input_data(j_decompress_ptr cinfo, long num_bytes);
void qimageio_term_source(j_decompress_ptr cinfo);
void qimageio_jpeg_src(j_decompress_ptr cinfo, QDataStream *image);

struct kimgio_jpeg_error_mgr {
	struct jpeg_error_mgr pub;
	jmp_buf stackContext;
};


//////
// Plug-in source manager for IJG JPEG compression/decompression library
//

//
// Control structures.
//

typedef struct {
	struct jpeg_source_mgr pub;        /* public fields */
	
	QDataStream *infile;               /* Pointer to QIODevice object */
	JOCTET * buffer;                   /* start of buffer */
	boolean start_of_file;             /* have we gotten any data yet? */
} qimageio_jpeg_source_mgr;


typedef struct {
	struct jpeg_destination_mgr pub;   /* public fields */

	QImageIO *iio;                     /* target stream */
	JOCTET buffer[OUTPUT_BUFFER_SIZE]; /* buffer */
} qimageio_jpeg_destination_mgr;


void
kimgio_jpeg_init_destination(j_compress_ptr cinfo)
{
	//Find out who we are. (Oh for classes!)
	qimageio_jpeg_destination_mgr *destMgr =
		(qimageio_jpeg_destination_mgr*)cinfo->dest;

	destMgr->pub.next_output_byte = destMgr->buffer;
	destMgr->pub.free_in_buffer = OUTPUT_BUFFER_SIZE;
}

int
kimgio_jpeg_empty_output_buffer(j_compress_ptr cinfo)
{
	//Find out who we are. (Oh for classes!)
	qimageio_jpeg_destination_mgr *destMgr =
		(qimageio_jpeg_destination_mgr*)cinfo->dest;
	QIODevice* ioDev = destMgr->iio->ioDevice();

	// Write the data to the imageio dev
	if (ioDev->writeBlock((char*)destMgr->buffer, OUTPUT_BUFFER_SIZE)
			!= OUTPUT_BUFFER_SIZE) {
		fprintf(stderr,"kimgio::jpeg::save write error\n");
		(*cinfo->err->error_exit)((j_common_ptr)cinfo);
	}
	destMgr->pub.next_output_byte = destMgr->buffer;
	destMgr->pub.free_in_buffer = OUTPUT_BUFFER_SIZE;

	return(true);
}


void
kimgio_jpeg_term_destination(j_compress_ptr cinfo)
{
	//Find out who we are. (Oh for classes!)
	qimageio_jpeg_destination_mgr *destMgr =
		(qimageio_jpeg_destination_mgr*)cinfo->dest;
	QIODevice* ioDev = destMgr->iio->ioDevice();

	// Write any bytes left in the buffer
	int bytesToWrite = OUTPUT_BUFFER_SIZE - destMgr->pub.free_in_buffer;

	// Write the data to the imageio dev
	if (ioDev->writeBlock((char*)destMgr->buffer, bytesToWrite) !=
			bytesToWrite) {
		fprintf(stderr,"kimgio::jpeg::save write error\n");
		(*cinfo->err->error_exit)((j_common_ptr)cinfo);
	}

	// Flush the qimagio buffer
	ioDev->flush();
	if (ioDev->status() != IO_Ok) {
		fprintf(stderr,"kimgio::jpeg::save write error\n");
		(*cinfo->err->error_exit)((j_common_ptr)cinfo);
	}
}

void
kimgio_jpeg_error_exit(j_common_ptr cinfo)
{
	fprintf(stderr, "kimgio:: jpeg error\n");
	/* Always display the message */
	(*cinfo->err->output_message) (cinfo);

	/* Let the memory manager delete any temp files before we die */
	jpeg_destroy(cinfo);

	longjmp(((kimgio_jpeg_error_mgr*)(cinfo->err))->stackContext, 1);
}


void kimgio_jpeg_write(QImageIO *iio)
{
	struct jpeg_compress_struct cinfo;
	struct kimgio_jpeg_error_mgr jerr;

	// add a proper error handler
	cinfo.err = jpeg_std_error((jpeg_error_mgr *)&jerr);
	jerr.pub.error_exit=kimgio_jpeg_error_exit;
	
	if (setjmp(jerr.stackContext)==0) { // will return here with ret==1 if error
		jpeg_create_compress(&cinfo);
		
		// add a destination manager
		qimageio_jpeg_destination_mgr destMgr;
		destMgr.iio=iio;
		destMgr.pub.init_destination=kimgio_jpeg_init_destination;
		destMgr.pub.empty_output_buffer=kimgio_jpeg_empty_output_buffer;
		destMgr.pub.term_destination=kimgio_jpeg_term_destination;
		cinfo.dest=(struct jpeg_destination_mgr *)&destMgr;
		
		QImage theImage=iio->image();
		int width=theImage.width();
		int height=theImage.height();
		
		int depth=theImage.depth();
		bool allGray=theImage.allGray();
		if (allGray)
			depth+=JPEG_GRAY;

		cinfo.image_width = width;
		cinfo.image_height = height;
		cinfo.input_components = allGray ? 1 : 3;
		cinfo.in_color_space = allGray ? JCS_GRAYSCALE : JCS_RGB;

		jpeg_set_defaults(&cinfo);
		/* Make optional parameter settings here */
		jpeg_set_quality(&cinfo, jpegQuality, true);
		cinfo.smoothing_factor=jpegSmoothing;
		cinfo.optimize_coding=jpegOptimizeCoding;

		jpeg_start_compress(&cinfo, TRUE);
		
		JSAMPROW row_pointer[1];        /* pointer to a single row */
		int row_stride;                 /* physical row width in buffer */	
		row_stride = width * cinfo.input_components;
		row_pointer[0]=new unsigned char[row_stride];
		
		QRgb *colourMap=theImage.colorTable();

		while (cinfo.next_scanline < cinfo.image_height) {
			unsigned char *colPos=row_pointer[0];
			switch(depth) {
			case 1:
				{
					unsigned char *theCol = (unsigned char*)
						theImage.scanLine(cinfo.next_scanline);
					if ( theImage.bitOrder() == QImage::LittleEndian ) {
						for(int pix=0;pix<width;pix++) {
							bool bit = *(theCol + (pix >> 3)) & (1 << (pix & 7));
							*colPos++ = qRed(colourMap[bit]);
							*colPos++ = qGreen(colourMap[bit]);
							*colPos++ = qBlue(colourMap[bit]);
						}
					} else {
						for(int pix=0;pix<width;pix++) {
							bool bit = *(theCol + (pix >> 3)) & (1 << (7 -(pix & 7)));
							*colPos++ = qRed(colourMap[bit]);
							*colPos++ = qGreen(colourMap[bit]);
							*colPos++ = qBlue(colourMap[bit]);
						}
					}

				}
				break;
				
			case 1+JPEG_GRAY:
				{
					unsigned char *theCol = (unsigned char*)
						theImage.scanLine(cinfo.next_scanline);
					if ( theImage.bitOrder() == QImage::LittleEndian ) {
						for(int pix=0;pix<width;pix++) {
							bool bit = *(theCol + (pix >> 3)) & (1 << (pix & 7));
							*colPos++ = qRed(colourMap[bit]);
						}
					} else {
						for(int pix=0;pix<width;pix++) {
							bool bit = *(theCol + (pix >> 3)) & (1 << (7 -(pix & 7)));
							*colPos++ = qRed(colourMap[bit]);
						}
					}
				}
				break;
				
			case 8:
				{
					unsigned char *theCol = (unsigned char*)
						theImage.scanLine(cinfo.next_scanline);
					for(int pix=0;pix<width;pix++) {
						*colPos++ = qRed(colourMap[*theCol]);
						*colPos++ = qGreen(colourMap[*theCol]);
						*colPos++ = qBlue(colourMap[*theCol]);
						theCol++;
					}
				}
				break;
				
			case 8+JPEG_GRAY:
				{
					unsigned char *theCol = (unsigned char*)
						theImage.scanLine(cinfo.next_scanline);
					for(int pix=0;pix<width;pix++) {
						*colPos++ = qRed(colourMap[*theCol]);
						theCol++;
					}
				}
				break;
				
			case 32:
				{
					QRgb* theCol = (QRgb*)theImage.scanLine(cinfo.next_scanline);
					for(int pix=0;pix<width;pix++) {
						*colPos++ = qRed(*theCol);
						*colPos++ = qGreen(*theCol);
						*colPos++ = qBlue(*theCol);
						theCol++;
					}
				}
				break;
				
			case 32+JPEG_GRAY:
				{
				QRgb* theCol = (QRgb*)theImage.scanLine(cinfo.next_scanline);
				for(int pix=0;pix<width;pix++)
					*colPos++ = qRed(*theCol++);
				break;
				}
			}
			jpeg_write_scanlines(&cinfo, row_pointer, 1);
		}
		delete row_pointer[0];
		
		jpeg_finish_compress(&cinfo);
		jpeg_destroy_compress(&cinfo);
		
		iio->setStatus(0);
	} else
		iio->setStatus(1);
}




///////////
//
// Plug-in to read files from JPEG into QImage
//

void kimgio_jpeg_read(QImageIO * iio)
{
	QIODevice *d = iio->ioDevice();
	QImage image;
	QDataStream s(d);
	JSAMPROW buffer[1];
	unsigned int *ui_row;
	unsigned char *uc_row, *uc_row_index;
	unsigned col;
	
	// Init jpeg decompression structures
	
	struct jpeg_decompress_struct cinfo;
	struct kimgio_jpeg_error_mgr jerr;

	// add a proper error handler
	cinfo.err = jpeg_std_error((jpeg_error_mgr *)&jerr);
	jerr.pub.error_exit=kimgio_jpeg_error_exit;
	
	if (setjmp(jerr.stackContext)==0) { // will return here with ret==1 if error
		jpeg_create_decompress(&cinfo);
		
		qimageio_jpeg_src(&cinfo, &s);
		jpeg_read_header(&cinfo, TRUE);

		jpeg_start_decompress(&cinfo);
		
		if (cinfo.out_color_space == JCS_GRAYSCALE) {
			image.create(cinfo.output_width, cinfo.output_height,
									 8, 256, QImage::LittleEndian);
	
			// Make colourmap
			for (col = 0; col < 256; col++)
				image.setColor(col, qRgb(col, col, col));
		} else {
			image.create(cinfo.output_width, cinfo.output_height, 32);
		}
		
		// Alloc one-row buffer for scanline
		buffer[0] = new JSAMPLE[cinfo.output_width * cinfo.output_components];
		
		//
		// Perform decompression
		//
		
		// Decompress grayscale
		if (cinfo.out_color_space != JCS_RGB) {
			while (cinfo.output_scanline < cinfo.output_height) {
				uc_row_index = image.scanLine(cinfo.output_scanline);
				uc_row = buffer[0];
				
				jpeg_read_scanlines(&cinfo, buffer, 1);
				
				for (col = 0; col < cinfo.output_width; col++)
					*uc_row_index++ = *uc_row++;
			}
		} else { // Decompress 24-bit RGB
			while (cinfo.output_scanline < cinfo.output_height) {
				ui_row = (unsigned int *)
					image.scanLine(cinfo.output_scanline);
				uc_row = buffer[0];
				
				jpeg_read_scanlines(&cinfo, buffer, 1);
				
				for (col = 0; col < cinfo.output_width; col++) {
					*ui_row++ = qRgb(*uc_row, *(uc_row+1), *(uc_row+2));
					uc_row+=3;
				}
			}
		}

		delete buffer[0];

		// Clean up JPEG structs
		
		jpeg_finish_decompress(&cinfo);
		jpeg_destroy_decompress(&cinfo);
		
		
		// Bind new image to screen
		
		iio->setImage(image);
		iio->setStatus(0);
	} else
		iio->setStatus(1);
}


/////////
//
// JPEG plug in routines for QDataStream as input.
//
// Registers stream routines for use by IJG JPEG library

void qimageio_jpeg_src(j_decompress_ptr cinfo, QDataStream * image)
{
	qimageio_jpeg_source_mgr *src;

	// Set up buffer for the first time

	if (cinfo->src == NULL) {

		cinfo->src = (struct jpeg_source_mgr *)
			(*cinfo->mem->alloc_small)
			((j_common_ptr) cinfo,
			 JPOOL_PERMANENT,
			 sizeof(qimageio_jpeg_source_mgr));

		src = (qimageio_jpeg_source_mgr *) cinfo->src;

		src->buffer = (JOCTET *)
			(*cinfo->mem->alloc_small)
			((j_common_ptr) cinfo,
			 JPOOL_PERMANENT,
			 INPUT_BUFFER_SIZE * sizeof(JOCTET));
	}
	// Register methods
	src = (qimageio_jpeg_source_mgr *) cinfo->src;

	src->pub.init_source = qimageio_init_source;
	src->pub.fill_input_buffer = qimageio_fill_input_buffer;
	src->pub.skip_input_data = qimageio_skip_input_data;
	src->pub.resync_to_restart = jpeg_resync_to_restart;
	src->pub.term_source = qimageio_term_source;

	// This is potentially dangerous, as it has chance of being
	// misinterpreted. Saves effort, tho!

	src->infile = image;

	src->pub.bytes_in_buffer = 0;	/* forces fill_input_buffer on first read */
	src->pub.next_input_byte = NULL;	/* until buffer loaded */

}


/////////
//
// Intializes data source
//

void qimageio_init_source(j_decompress_ptr cinfo)
{
	qimageio_jpeg_source_mgr *ptr = (qimageio_jpeg_source_mgr *) cinfo->src;

	ptr->start_of_file = TRUE;
}	



/////////
//
// Reads data from stream into JPEG working buffer
//

int qimageio_fill_input_buffer(j_decompress_ptr cinfo)
{
	qimageio_jpeg_source_mgr *ptr = (qimageio_jpeg_source_mgr *) cinfo->src;
	size_t nbytes;

	nbytes = ptr->infile->device()->readBlock((char *) (ptr->buffer),
																						INPUT_BUFFER_SIZE);

	if (nbytes <= 0) {

		if (ptr->start_of_file) {
			fprintf(stderr, "error: file empty.\n");

			return FALSE;
		}
		fprintf(stderr, "warning: premature EOF in file.\n");

		/* Insert a fake EOI marker */
		ptr->buffer[0] = (JOCTET) 0xFF;
		ptr->buffer[1] = (JOCTET) JPEG_EOI;
		nbytes = 2;

	}
	ptr->pub.next_input_byte = ptr->buffer;
	ptr->pub.bytes_in_buffer = nbytes;
	ptr->start_of_file = FALSE;

	return TRUE;
}



//////////
//
// Jumps over large chunks of data. Can probably be done quicker. (fseek()
// style).

void qimageio_skip_input_data(j_decompress_ptr cinfo, long num_bytes)
{
	qimageio_jpeg_source_mgr *ptr = (qimageio_jpeg_source_mgr *) cinfo->src;


	if (num_bytes > 0) {

		while (num_bytes > (long) ptr->pub.bytes_in_buffer) {
			num_bytes -= (long) ptr->pub.bytes_in_buffer;
			(void) qimageio_fill_input_buffer(cinfo);
		} ptr->pub.next_input_byte += (size_t) num_bytes;
		ptr->pub.bytes_in_buffer -= (size_t) num_bytes;
	}
}



/////////
//
// Clean up. Doesn't really do anything, but required for compat.
//

void qimageio_term_source(j_decompress_ptr)
{
	return;
}




//=====================================================

#include <math.h>

#define MAX(A,B) (((A)>(B)) ? (A) : (B))

QImage
readJPEGThumbnail(QString fileName, QSize sz)
{
	QImage image;
	FILE * infile;
	int realWidth, realHeight;

	JSAMPROW buffer[1]={0};
	unsigned int *ui_row;
	unsigned char *uc_row, *uc_row_index;
	unsigned col;
	
	// Init jpeg decompression structures
	
	struct jpeg_decompress_struct cinfo;
	struct kimgio_jpeg_error_mgr jerr;

	// add a proper error handler
	cinfo.err = jpeg_std_error((jpeg_error_mgr *)&jerr);
	jerr.pub.error_exit=kimgio_jpeg_error_exit;

	if (setjmp(jerr.stackContext)==0) { // will return here with ret==1 if error
		jpeg_create_decompress(&cinfo);
		
		if ((infile = fopen(fileName, "rb")) == NULL)
			return(image);

		jpeg_stdio_src(&cinfo, infile);
		jpeg_read_header(&cinfo, TRUE);

		realWidth=cinfo.image_width;
		realHeight=cinfo.image_height;

		float s=MAX((float(realWidth)/sz.width()),
					(float(realHeight)/sz.height()));
		//fprintf(stderr, "scale=%f\n",s);
		if (s>8)
			cinfo.scale_denom=8;
		else
			cinfo.scale_denom=(int)pow(2,floor(log(s)/log(2.)));
		//fprintf(stderr, "cinfo.scale_denom=%d\n", cinfo.scale_denom);
				
		jpeg_start_decompress(&cinfo);
		
		if (cinfo.out_color_space == JCS_GRAYSCALE) {
			image.create(cinfo.output_width, cinfo.output_height,
									 8, 256, QImage::LittleEndian);

			// Make colourmap
			for (col = 0; col < 256; col++)
				image.setColor(col, qRgb(col, col, col));
		} else {
			image.create(cinfo.output_width, cinfo.output_height, 32);
		}
				
		//
		// Perform decompression
		//
		
		// Alloc one-row buffer for scanline
		buffer[0] = new JSAMPLE[cinfo.output_width * cinfo.output_components];

		// Decompress grayscale
		if (cinfo.out_color_space != JCS_RGB) {
			while (cinfo.output_scanline < cinfo.output_height) {
				uc_row_index = image.scanLine(cinfo.output_scanline);
				uc_row = buffer[0];
				
				jpeg_read_scanlines(&cinfo, buffer, 1);
				
				for (col = 0; col < cinfo.output_width; col++)
					*uc_row_index++ = *uc_row++;
			}
		} else { // Decompress 24-bit RGB
			while (cinfo.output_scanline < cinfo.output_height) {
				ui_row = (unsigned int *)
					image.scanLine(cinfo.output_scanline);
				uc_row = buffer[0];
				
				jpeg_read_scanlines(&cinfo, buffer, 1);
				
				for (col = 0; col < cinfo.output_width; col++) {
					*ui_row++ = qRgb(*uc_row, *(uc_row+1), *(uc_row+2));
					uc_row+=3;
				}
			}
		}
	

		// Clean up JPEG structs
		
		jpeg_finish_decompress(&cinfo);
		jpeg_destroy_decompress(&cinfo);
		fclose(infile);

		if (!image.isNull())
			image=image.smoothScale(int(realWidth/s),	int(realHeight/s));
	} else
		image=QImage();

	if (buffer[0])
		delete buffer[0];
	
	return(image);
}


//====================================================================

}
#endif
