//
// Copyright (c) 2000-2001 - Andrew Richards <ajr@users.sourceforge.net>
//
// This file is part of the PikView image viewer released under the GNU GPL.
// For license details see the file COPYING included with this distribution.
//

#include "qlistviewdrag.h"
#include "selector.h"
#include <qdragobject.h>
#include <stdio.h>

QListViewDrag::QListViewDrag(QWidget *parent=0, const char *name=0)
	: QListView(parent, name)
{
	startDrag=false;
	setAcceptDrops(true);
	blockDrops=false;
}

void
QListViewDrag::contentsMousePressEvent(QMouseEvent *e)
{
	QListViewItem *item=itemAt(e->pos());
	if (item && (item->isSelected()) && (e->button()==LeftButton) && (e->state()==0)) { // NB state==LB => no modifiers	
		startDrag=true;
		startDragPos=e->pos();
	} else {
		startDrag=false;
		QListView::contentsMousePressEvent(e);
	}
}


void
QListViewDrag::contentsMouseReleaseEvent(QMouseEvent *e)
{
	if (startDrag) {
		QMouseEvent ev(QEvent::MouseButtonPress, startDragPos, LeftButton, 0);
		QListView::contentsMousePressEvent(&ev);
		startDrag=false;
	}
	QListView::contentsMouseReleaseEvent(e);
}
	

void
QListViewDrag::contentsMouseMoveEvent(QMouseEvent *e)
{
	if (startDrag) {
		if ((e->pos()-startDragPos).manhattanLength() > QApplication::startDragDistance()) {
			QListViewItem *item=itemAt(e->pos());
			if (item && item->isSelected()) {
				QStrList uris;
				selectorItem *it=(selectorItem*)firstChild();
				for(; it; it=(selectorItem*)it->itemBelow()) {
					if (it->isSelected())
						uris.append(QUriDrag::localFileToUri(it->filePath()));
				}
				
// 				for(char *i=uris.first(); i; i=uris.next())
// 					printf("item %s\n",i);
				
				bool acceptTemp=acceptDrops();
				setAcceptDrops(false); // cant have src and dest of drop the same
				QDragObject *d=new QUriDrag(uris, this);
				//printf("dragging\n");
				d->drag();
				setAcceptDrops(acceptTemp);
				startDrag=false;
				//printf("dragged\n");
				return;
			} else
				puts("QListViewDrag::contentsMouseMoveEvent - This shouldn't happen");
		}
		return;
	}
	QListView::contentsMouseMoveEvent(e);
}


void
QListViewDrag::dragEnterEvent(QDragEnterEvent *e)
{	
	e->accept(QUriDrag::canDecode(e));
}

void
QListViewDrag::dropEvent(QDropEvent *e)
{
	//puts("dropEvent");
	emit dropped(e);
}
