//
// Copyright (c) 2000-2001 - Andrew Richards <ajr@users.sourceforge.net>
//
// This file is part of the PikView image viewer released under the GNU GPL.
// For license details see the file COPYING included with this distribution.
//


#include <qlayout.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qcheckbox.h>
#include <qtable.h>
#include <kfiledialog.h>
#include <klocale.h>

#include "rapidFileConfig.h"
#include "options.h"
#include "misc.h"


rapidFileConfig::rapidFileConfig(QWidget *parent=0, const char *name=0)
	: QDialog(parent, name, true)
{
	setCaption(i18n("PikView")+":"+i18n("Shortcut configuration"));

	QBoxLayout *lyMain=new QVBoxLayout(this);
	QBoxLayout *lyList=new QHBoxLayout;
	lyMain->addLayout(lyList,1);

	list=new QTable(this);
	list->setSelectionMode(QTable::NoSelection);
	list->setNumCols(2);
	list->horizontalHeader()->setLabel(0, i18n("Key seq."), 80);
	list->horizontalHeader()->setLabel(1, i18n("Directory"), 280);
	list->setMinimumWidth(400);
	lyList->addWidget(list,1);

	QFrame *frame=new QFrame(this);
	frame->setFrameStyle( QFrame::VLine | QFrame::Sunken );
	frame->setFixedWidth(15);
	lyList->addWidget(frame);

	QBoxLayout *lyButt=new QVBoxLayout;
	lyList->addLayout(lyButt);
	lyButt->addStretch(1);
	btNew=new QPushButton(i18n("New"), this);
	btNew->setFixedSize(btNew->sizeHint());
	lyButt->addWidget(btNew);
	btDel=new QPushButton(i18n("Del"), this);
	btDel->setFixedSize(btNew->sizeHint());
	lyButt->addWidget(btDel);
	btDir=new QPushButton(i18n("Dir.."), this);
	btDir->setFixedSize(btNew->sizeHint());
	lyButt->addWidget(btDir);
	lyButt->addStretch(1);


	frame=new QFrame(this);
	frame->setFrameStyle( QFrame::HLine | QFrame::Sunken );
	frame->setFixedHeight(15);
	lyMain->addWidget(frame);

	QBoxLayout *lyOkCan=new QHBoxLayout;
	lyMain->addLayout(lyOkCan,0);
	cbEnable=new QCheckBox(i18n("Enable filing shortcuts"), this);
	cbEnable->setFixedSize(cbEnable->sizeHint());
	cbEnable->setChecked(opt.enableFiling);
	lyOkCan->addWidget(cbEnable);
	lyOkCan->addStretch(1);
	btOk=new QPushButton(i18n("Okay"), this);
	btOk->setFixedSize(btOk->sizeHint());
	lyOkCan->addWidget(btOk);
	lyOkCan->addSpacing(10);
	btCan=new QPushButton(i18n("Cancel"), this);
	btCan->setFixedSize(btCan->sizeHint());
	lyOkCan->addWidget(btCan);
	lyOkCan->addStretch(1);

	char *sc;
	int items=0;
	for(sc=opt.shortCuts.first(); sc; items++, sc=opt.shortCuts.next()) {
		addEntry();
		list->setText(items, 0, sc);
		list->setText(items, 1, opt.shortCutDirs.at(items));
	}

	connect(btDel, SIGNAL(clicked()), this, SLOT(deleteEntry()));
	connect(btNew, SIGNAL(clicked()), this, SLOT(addEntry()));
	connect(btDir, SIGNAL(clicked()), this, SLOT(chooseDir()));

	connect(btOk,  SIGNAL(clicked()), this, SLOT(okPressed()));
	connect(btCan, SIGNAL(clicked()), this, SLOT(cancelPressed()));

	resize(460, 350);
}


void
rapidFileConfig::addEntry()
{
	list->setNumRows(list->numRows()+1);
	QTableItem *item=new QTableItem(list, QTableItem::WhenCurrent, "");
	list->setItem(list->numRows()-1, 0, item);
	item=new QTableItem(list, QTableItem::WhenCurrent, "");
	list->setItem(list->numRows()-1, 1, item);
}

void
rapidFileConfig::deleteEntry()
{
	int cr=list->currentRow();
	for(int r=cr; r<list->numRows()-1; r++) {
		list->swapRows(r,r+1);
	}
	list->setNumRows(list->numRows()-1);
	list->setCurrentCell(cr, 0);
}


void
rapidFileConfig::chooseDir()
{
	if (list->numRows()>0) {
		list->setCurrentCell(list->currentRow(), 0);
 		QString dir=KFileDialog::getExistingDirectory(list->text(list->currentRow(), 1));
 		if (!dir.isEmpty()) {
 			if (dir.right(1)=="/")
 				dir=dir.left(dir.length()-1);
 			list->setText(list->currentRow(), 1, dir);
 		}
	}
}


void
rapidFileConfig::cancelPressed()
{
	reject();
}


void
rapidFileConfig::okPressed()
{
	int l1, l2, l;

	opt.enableFiling=cbEnable->isChecked();
	
 	int rows=list->numRows();
 	for(int r=0; r<rows-1; r++) {
		if (list->text(r, 0).isEmpty())
			continue;
 		for(int i=r+1; i<rows; i++) {
			if (list->text(i, 0).isEmpty())
 				continue;
 			l1=list->text(r, 0).length();
 			l2=list->text(i, 0).length();
 			if (l1!=l2) {
 				l=MIN(l1,l2);
 				if (list->text(r, 0).left(l) == list->text(i, 0).left(l)) {
 					QMessageBox::information(this, i18n("Conflict"),
						i18n("You cannot have both \"%1\" and \"%2\" as shortcuts "
								 "because \"%3\" would have priority"
								 ).arg(list->text(r, 0)).arg(list->text(i, 0)).arg(list->text(r, 0).left(l)),
																	 "OK");
					return;
				}
			}
			if (list->text(r, 0)==list->text(i, 0)) {
				QMessageBox::information( this, i18n("Conflict"),
																	i18n("Duplicate shortcut \"%1\"").arg(list->text(r, 0)), i18n("OK"));
				return;
			}
		}
	}

	opt.shortCuts.clear();
	opt.shortCutDirs.clear();
	for(int r=0; r<rows; r++) {
		if (!list->text(r, 0).isEmpty()) {
			opt.shortCuts.append(list->text(r,0));
			opt.shortCutDirs.append(list->text(r,1));
		}
	}
	opt.saveShortCuts();

	accept();
}


void
rapidFileConfig::closeEvent(QCloseEvent *e)
{
	okPressed();
}
