//
// Copyright (c) 2000-2001 - Andrew Richards <ajr@users.sourceforge.net>
//
// This file is part of the PikView image viewer released under the GNU GPL.
// For license details see the file COPYING included with this distribution.
//

#include <unistd.h>
#include <qfileinfo.h>
#include <klocale.h>
#include "selector.h"
#include "thumbWidget.h"
#include "options.h"
#include "viewer.h"

void
selector::renameItem(QIconViewItem *item)
{
	((thumbWidgetItem*)item)->item->setText(0,item->text());
	renameItem(((thumbWidgetItem*)item)->item);
}


void
selector::renameItem(QListViewItem *item)
{
	selectorItem *theItem=(selectorItem*)item;
	if (theItem->fileName()==theItem->text(0))
		return;
	//printf("rename %s -> %s\n",theItem->fileName().ascii(), theItem->text(0).ascii());
	if (theItem->text(0).find("/")!=-1) {
		QMessageBox::warning( this, i18n("PikView"), i18n("You cannot use / in a filename."));
		theItem->setText(0, theItem->fileName());
		if (theItem->thumbItem)
			theItem->thumbItem->setText(theItem->fileName());
		return;
	}
	QDir dir(theItem->fileDir());
	if (dir.rename(theItem->fileName(), theItem->text(0))) {
		// update the fileInfo for this item
		//puts("renamed");
		QFileInfo f(theItem->fileDir()+"/"+theItem->text(0));
		theItem->setInfo(&f);
	} else {
		QMessageBox::warning(this, i18n("PikView"), 
												 i18n("This item cannot be renamed to \"%1\".").arg(theItem->text(0)));
		theItem->setText(0, theItem->fileName());
		if (theItem->thumbItem)
			theItem->thumbItem->setText(theItem->fileName());
		return;
	}
}

void
selector::slotDeleteKey()
{
	QList<selectorItem> items;
	
	if (opt.fileViewType==0) {
		selectorItem *next;
		for(selectorItem *item=(selectorItem *)fileList->firstChild(); item; item=next) {
			next=(selectorItem *)item->nextSibling();
			if (item->isSelected())
				items.append(item);
		}	
	} else {
		QIconViewItem *next;
		for(QIconViewItem *item=thumbNails->firstItem(); item; item=next) {
			next=item->nextItem();
			if (item->isSelected())
				items.append(((thumbWidgetItem*)item)->item);
		}
	}
	deleteItems(items);	
}


void
selector::deleteItems(QList<selectorItem> &items)
{
	if (items.count()==0)
		return;

	int res;
	QString mess;
	if (opt.confirmDelete) {
		if (items.count()==1)
			mess=i18n("Are you sure you want to delete: %1?").arg(items.first()->text(0));
		else
			mess.sprintf(i18n("Are you sure you want to delete %d items?"), items.count());
		res=QMessageBox::warning(0, i18n("PikView"), mess, i18n("Ok"), i18n("Cancel"));
		if (res!=0) return;
	}
	// Stop updates while Delete in progesss
	// (only need this if using and event loop based deleter (eg KIO::Netaccess:del)
	killTimer(dirWatchTimer);
	killTimer(fileWatchTimer);
	
	for(selectorItem *item=items.first(); item; item=items.next()) {
		printf("del %s\n",item->filePath().ascii());
		//=KIO::NetAccess::del(KURL(item->filePath()));  // This is very slow
		res=unlink(item->filePath());
		if (res) {
			mess=i18n("Unable to delete file: %1. Continue?").arg(item->filePath());
			res=QMessageBox::warning(this, i18n("PikView"), mess, i18n("Ok"), i18n("Cancel"));
			if (res!=0) return;
		} else
			removeItem(item);
	}
	dirWatchTimer=startTimer(100);
	fileWatchTimer=startTimer(5000);
	
	if (opt.fileViewType==1)
		thumbNails->arrangeItemsInGrid(true);
	updateItemCount();
}


void
selector::slotDeleteCurrentImage()
{
	if (currentFileItem) {
		QList<selectorItem> items;
		items.append(currentFileItem);
		deleteItems(items);
		
		if (currentFileItem)
			goImage(0);
		else
			theViewer->goSplashImage();			
	}
}
