//
// Copyright (c) 2000-2001 - Andrew Richards <ajr@users.sourceforge.net>
//
// This file is part of the PikView image viewer released under the GNU GPL.
// For license details see the file COPYING included with this distribution.
//

#include "syncCopyJob.h"
#include <qprogressdialog.h>
#include <stdio.h>
#include <qapp.h>
#include <klocale.h>

SyncCopyJob::SyncCopyJob(const KURL::List& src, const KURL& dest, KIO::CopyJob::CopyMode mode, QWidget *parent)
	: QObject()
{
	progress=new QProgressDialog(parent, "SyncCopyJobProgess", true);
	fileCount=0;
	totalFileCount=src.count();
	progress->setTotalSteps(totalFileCount);
	//progress->setMinimumDuration(1500);

	progress->setLabelText((mode==KIO::CopyJob::Copy) ? i18n("Copying files...") : i18n("Moving files..."));
	
	job=new KIO::CopyJob(src, dest, mode, false, false);
	
	connect(job,  		SIGNAL(result(KIO::Job*)),
					this, 		SLOT(jobResult(KIO::Job*)));

 	connect(job,      SIGNAL(processedFiles(KIO::Job*, unsigned long)),
 					this,		 	SLOT(processedFilesSlot(KIO::Job*, unsigned long)));
}

SyncCopyJob::~SyncCopyJob()
{
	// apparantly job deletes itself
 	if (progress) delete progress;
}

void
SyncCopyJob::exec()
{
	finished=false;

	while(!finished && !progress->wasCancelled())
		qApp->processEvents();

	if (progress->wasCancelled() && !finished) {
		job->kill();
		printf("Cancelled\n");
	}
 	delete progress;
	progress=0;
}

void
SyncCopyJob::jobResult(KIO::Job *j)
{
	// do something with a non zero j->error()
	//printf("Job result = %d (%s)\n", j->error(), j->errorString().ascii());
	finished=true;
}


void
SyncCopyJob::processedFilesSlot(KIO::Job *job, unsigned long files)
{
	fileCount+=files;
	//printf("processed files=%ld\n", fileCount);
	progress->setProgress(fileCount);
}
