//
// Copyright (c) 2000-2001 - Andrew Richards <ajr@users.sourceforge.net>
//
// This file is part of the PikView image viewer released under the GNU GPL.
// For license details see the file COPYING included with this distribution.
//


#ifndef THUMBWIDGET_H
#define THUMBWIDGET_H

#include <qwidget.h>
#include <qstring.h>
#include <qlist.h>
#include <qpixmap.h>
#include <qiconview.h>
#include <qimage.h>

class thumbWidget;
class selectorItem;
class QListViewItem;

// =======================================================================
// Class thumbWidgetItemDrag

class thumbWidgetItemDrag : public QIconDrag
{
	Q_OBJECT

public:
	thumbWidgetItemDrag(QWidget *dragSource, const char *name=0);

	const char* format(int i) const;
	QByteArray encodedData(const char *mime) const;
	static bool canDecode(QMimeSource *e);
	void append(const QIconDragItem &item, const QRect &pr, const QRect &tr, const QString &url);

private:
	QStringList urls;
};



// =======================================================================
// Class thumbWidgetItem

class thumbWidgetItem : public QIconViewItem
{
 public:
	thumbWidgetItem(QIconView * parent, selectorItem *selItem);

	selectorItem *item;
};



// =======================================================================
// Class thumbWidget

class thumbWidget : public QIconView
{
	Q_OBJECT;

 public:
	thumbWidget(QWidget *parent=0, const char *name=0, int wFlags=0);
	~thumbWidget();

	void setImageList(selectorItem *firstItem);
	void removeItem(thumbWidgetItem *item=0);
	void updateThumbNails();
	virtual void clear();

 public slots:
	virtual void show();
  virtual void hide();

 private slots:
  void updateCellSize(int newX, int newY);
  void slotCurrentChanged(QIconViewItem *item);
	void slotDoubleClicked(QIconViewItem *item);	

 protected:
	virtual void timerEvent(QTimerEvent *);
	virtual QDragObject *dragObject();

 signals:
	void doubleClicked(QListViewItem*);

 private:
	int processTimer, timerDelay;
	int maxThumbSizeX, maxThumbSizeY;
	int selectionCount;
	QIconViewItem *thumbCell;
};

#endif
