//
// Copyright (c) 2000-2001 - Andrew Richards <ajr@users.sourceforge.net>
//
// This file is part of the PikView image viewer released under the GNU GPL.
// For license details see the file COPYING included with this distribution.
//


#ifndef VIEWER_H
#define VIEWER_H

#include <qwidget.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qstrlist.h>
#include <qdir.h>
#include <qrect.h>
#include <pthread.h>

class viewer;
class selector;
class selectorItem;

#include "types.h"


class viewer : public QWidget
{
	friend class options;
	friend class optionsDialog;

	Q_OBJECT;
 public:
	viewer(QStrList *files, QWidget *parent=0, const char *name=0,
				 int wFlags=0);
	~viewer();

	QPixmap  currentPicture();
	bool     slideShowMode() { return slideShowRunning; };
	QString  currentFile()   { return currentPath;      };

 public slots:
	void newImage(QPixmap, QString imagePath);
	void setFullScreen(enum logicChoice set);
	virtual void hide();

	void rotateLeft();
	void rotateRight();
	void rotate180();
	void mirrorX();
	void mirrorY();
	void showMessage(QString mess, int delay=2000);
	void goSplashImage();

 protected:
	virtual void paintEvent( QPaintEvent * );
	virtual void mousePressEvent( QMouseEvent * );
	virtual void mouseReleaseEvent( QMouseEvent * );
	virtual void mouseMoveEvent( QMouseEvent * );
	virtual void wheelEvent( QWheelEvent * );
	virtual void keyPressEvent( QKeyEvent * );
	virtual void timerEvent(QTimerEvent *e);
 	virtual void closeEvent(QCloseEvent *e);
	virtual	void resizeEvent( QResizeEvent * );
	virtual void dragEnterEvent(QDragEnterEvent* event);
	virtual void dropEvent(QDropEvent* event);
	virtual void focusInEvent(QFocusEvent *);
	virtual void focusOutEvent(QFocusEvent *);
	virtual void mouseDoubleClickEvent(QMouseEvent *e);

 signals:
	void imageSelectSignal(int);
	void newScale(float scaleFact);
	void setSelectorVisibile(enum logicChoice);
	void viewerClosed();
	void refileImage(QString, QString);
	void viewerDropEvent(QDropEvent *event);
	void deleteCurrentImage();

 private:
	void loadSettings();
	void newImageSetup();
	void panImage(int px, int py);
	void scaleImage(enum moveDir dir, float scale=1);
	void setHideMouse(enum logicChoice set);
	void showRect(char *name, QRect r);
	void showPoint(char *name, QPoint p);
	void scaleSizeRelCenter(QRect &r, float c);
	void setCentering(enum logicChoice set);
	void setSlideShow(enum logicChoice set);
	void slideShowNext();
	void constrainBitmapPos();
	void doContextMenu();
	void showInfo(int delay=2000);
	void processShortcut(int ch);
	void funkyZoom();

	QPixmap currentImage;
	QString currentPath, currentType, currentInfo, currentMessage;
	QString shortcut, oldShortcut;

	selectorItem *currentImageItem, *lookAheadItem;
	int lookAheadNumber, updateTimer, slideShowTimer, bufferedBytes, funkyTimer,
		maxLookAhead, slideShowRunning, infoTimer, messageTimer, bitmapPanStep;
	QRect  infoBox, messageBox;
	float  scaleFact;
	QRect  winRect;
	QPoint frameTopLeft, imagePos, bitmapPos;
	float dragStartPosX, dragStartPosY;
	QSize  frameSize;
	bool blockPaintEvents;
};

#endif
